import scrapy
import mysql.connector
from mysql.connector import errorcode
from ..items import NewsItem
from ..settings import MYSQL_USERNAME, MYSQL_PASSWORD, MYSQL_HOST, MYSQL_PORT, MYSQL_DATABASE

class BjxpvSpider(scrapy.Spider):
    name = "bjxpv"
    allowed_domains = ["guangfu.bjx.com.cn"]


    def __init__(self, *args, **kwargs):
        super(BjxpvSpider, self).__init__(*args, **kwargs)
        self.crawled_urls = set()


    def start_requests(self):
        # 连接数据库
        self.conn = mysql.connector.connect(user=MYSQL_USERNAME, password=MYSQL_PASSWORD, host=MYSQL_HOST, database=MYSQL_DATABASE, port=MYSQL_PORT)
        self.cursor = self.conn.cursor()
        
        # 动态生成表名
        dataset_name = f'{self.settings.get("BOT_NAME")}_{self.name}'
        
        # 获取当前数据库中已经爬取的 URLs
        try:
            self.cursor.execute(f"SELECT url FROM `{dataset_name}`")
            self.crawled_urls = {row[0] for row in self.cursor.fetchall()}
        except mysql.connector.Error as err:
            if err.errno == errorcode.ER_NO_SUCH_TABLE:
                self.log(f"Table `{dataset_name}` does not exist. Initializing crawled URLs as an empty set.")
                self.crawled_urls = set()
            else:
                self.log(f"Error fetching URLs from `{dataset_name}`: {err}")
                self.crawled_urls = set()
        
        # 断开数据库连接
        self.conn.close()

        # 开始请求
        start_urls = [
            'https://guangfu.bjx.com.cn/yw/',
            'https://guangfu.bjx.com.cn/zc/',
            'https://guangfu.bjx.com.cn/sc/',
            'https://guangfu.bjx.com.cn/mq/',
            'https://guangfu.bjx.com.cn/dj/',
            'https://guangfu.bjx.com.cn/xm/',
            'https://guangfu.bjx.com.cn/zb/',
            'https://guangfu.bjx.com.cn/cj/',
            'https://guangfu.bjx.com.cn/gj/',
            'https://guangfu.bjx.com.cn/sj/',
            'https://guangfu.bjx.com.cn/js/',
        ]
        for url in start_urls:
            yield scrapy.Request(url, self.parse)


    def parse(self, response):
        news_list = response.css('.cc-list-content a')
        for news in news_list:
            url = news.attrib['href']
            yield scrapy.Request(url, self.news_parse)

        next_page = response.xpath('//a[contains(text(), "下一页")]')
        if next_page.attrib.get('class') != 'disable':
            url = next_page.attrib['href']
            yield response.follow(url, self.parse)


    def news_parse(self, response):
        news_item = NewsItem()
        news_item['website'] = '北极星太阳能光伏网'
        news_item['url'] = response.url
        news_item['collection'] = ';'.join([c.get() for c in response.xpath('//div[@class="cc-crumbs"]/div[@class="box"]/em/a/text()')])

        news_item['title'] = response.xpath('//div[@class="cc-headline"]/div[@class="box"]/h1/text()').get()
        news_item['date'] = response.xpath('//div[@class="cc-headline"]/div[@class="box"]/p/span/text()').re(r'(\d{4}-\d{2}-\d{2})')[0]
        news_item['source'] = response.xpath('//div[@class="cc-headline"]/div[@class="box"]/p/span[2]/text()').re(r'来源：(.*)')[0]
        news_item['keywords'] = ';'.join([k.get() for k in response.xpath('//span[@id="key_word"]/em/a/text()')])
 
        paragraphs = response.css('.cc-article p')
        news_item['content'] = '\n'.join([''.join([c.get() for c in p.xpath('.//text()')]) for p in paragraphs])

        yield news_item
